// RBC model 
// with variable labor

var cc, ccn, cch, hw, hwn, hwt, eta
    lambda, gamman, gammat, gdp, yn, yt, 
    j, jn, jt, in, ih, kpn, kpt, 
    wn, wt, rn, rt, nn, nt, pc, 
    pj, pn, sy, jy;
    
//varexo nnerr;
varexo nnerr, nterr;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// % define parameters here
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

parameters beta_p, chi_p, hw_p, delta_p, alphan_p, alphat_p, 
           ss_p, r_p, sn_p, st_p, yn2y_p,yt2y_p, hwt_p, hwn_p, y_p, yt_p, yn_p, el,
           chi0_p, rhonn_p, rhont_p, omegacn_p, omegact_p,
           omegain_p, omegait_p, rhon_p, rhot_p psik_p;


// Discount factor  (default .99)
beta_p = 0.99;

// Frisch Elasticity of labor supply (2/chi_p);
chi_p = 10;

// Steady state labor supply
hw_p = 0.5;

// depreciation rate (default 0.025)
delta_p = 0.025;
 
// Capital share
alphan_p = 0.3;
alphat_p = 0.3;

// consumption and investment weights
omegacn_p = 0.5;

omegain_p = 0.5;


// Rental rate for capital in both sectors
r_p = (delta_p - 1 + 1/beta_p);

// sectoral saving rate
sn_p = alphan_p * delta_p/r_p;
st_p = alphat_p * delta_p/r_p;

// relative size of sectors
yn2y_p = (omegacn_p*(1-st_p) + omegain_p*st_p) /(1-omegacn_p*(st_p - sn_p)-omegain_p*(sn_p-st_p));
yt2y_p = 1-yn2y_p;

// aggregate saving rate
ss_p = yn2y_p*sn_p + yt2y_p*st_p;

// output
y_p = hw_p/( yt2y_p * (delta_p/st_p)^(alphat_p/(1-alphat_p)) + yn2y_p * (delta_p/sn_p)^(alphan_p/(1-alphan_p)) );
yt_p = yt2y_p * y_p;
yn_p = yn2y_p * y_p;

// sectoral labor
hwt_p = yt2y_p*(delta_p/st_p)^(alphat_p/(1-alphat_p))*y_p;
hwn_p = yn2y_p*(delta_p/sn_p)^(alphan_p/(1-alphan_p))*y_p;

chi0_p = (1-hw_p)^(chi_p)*(1/((1-ss_p)*y_p)) *(1-alphat_p)*(yt_p/hwt_p);

el = 2;
rhot_p = 1/(el-1);

el = 2;
rhon_p = 1/(el-1);

psik_p = 0.5;

rhonn_p = 0.995;    
rhont_p = 0.995;
  
model;
///////////////////////////////////////////////////////////////////////////
// Country 1
///////////////////////////////////////////////////////////////////////////

// 1. FOC with respect to consumption
1/cc - lambda*pc=0;   

// 2. FOC wrt to aggregate labor
-chi0_p*(1-hw)^(-chi_p) +eta=0;

// 3-4. FOC wrt to labor in each sector
lambda*wn*pn-eta = 0;
lambda*wt-eta = 0;

// 5. FOC with respect to investment
gamman-lambda*pj -gamman*psik_p*(jn/kpn(-1)-delta_p) = 0;
gammat-lambda*pj -gamman*psik_p*(jt/kpt(-1)-delta_p) = 0;


// 7. gamman shadow price of capital
(1-delta_p)*beta_p * gamman(1) - gamman + beta_p*pn(1)*lambda(1)*rn(1)+beta_p*gamman(1)*psik_p*(jn(1)/kpn-delta_p)/kpn = 0;
(1-delta_p)*beta_p * gammat(1) - gammat + beta_p*lambda(1)*rt(1)+beta_p*gammat(1)*psik_p*(jt(1)/kpt-delta_p)/kpt = 0;

// 9. technology constraints
yn = kpn(-1)^alphan_p*(nn*hwn)^(1-alphan_p);
yt = kpt(-1)^alphat_p*(nt*hwt)^(1-alphat_p);

// 11. definition of aggregate output
gdp = pn*yn+yt;

// 12. resource constraints
yn = ccn + in;
yt = cch + ih;
j = jt + jn;
hw = hwt + hwn;

// 16. definition of consumption aggregator 
//cc  = (omegacn_p^(rhon_p/(1+rhon_p))*ccn^(1/(1+rhon_p))
//     +(1-omegacn_p)^(rhon_p/(1+rhon_p))*cch^(1/(1+rhon_p)))^(1+rhon_p);
pc = (omegacn_p*pn^(-1/rhon_p)+(1-omegacn_p))^(-rhon_p);

// 18. demand equations
ccn = omegacn_p*(pn/pc)^(-(1+rhon_p)/rhon_p) *cc;
cch = (1-omegacn_p)*(1/pc)^(-(1+rhon_p)/rhon_p) *cc;

// 22. definition of consumption aggregator (pins down ccd)
//j =  (omegain_p^(rhon_p/(1+rhon_p))*in^(1/(1+rhon_p))+(1-omegain_p)^(rhon_p/(1+rhon_p))*ih^(1/(1+rhon_p)))^(1+rhon_p);

pj = (omegain_p*pn^(-1/rhon_p)+(1-omegain_p))^(-rhon_p);

// 24. demand equations
in = omegain_p*(pn/pj)^(-(1+rhon_p)/rhon_p) *j;
ih = (1-omegain_p)*(1/pj)^(-(1+rhon_p)/rhon_p) *j;


// 31. Saving rate
sy = 1-cc/gdp;

// 32. Investment rate
jy = j/gdp;

// 36. Capital
kpt = (1-delta_p)*kpt(-1) + jt;
kpn = (1-delta_p)*kpn(-1) + jn;

// 38. Rental Rate for Labor, FOC with respect to labor
wn = (1-alphan_p)*(yn/(hwn*nn))*nn;
wt = (1-alphat_p)*(yt/(hwt*nt))*nt;

// 40. Rental Rate for Capital
rn = alphan_p*(yn/kpn(-1));
rt = alphat_p*(yt/kpt(-1));

// 42. Labor-augmenting technology shock
nn-1 = rhonn_p*(nn(-1)-1) + nnerr;
nt-1 = rhont_p*(nt(-1)-1) + nterr;




end;




initval;

///////////////////////////////////////////////////////////////////////////
// Steady state for  country 1
///////////////////////////////////////////////////////////////////////////


// Rental rate for capital
rn = r_p;
rt = r_p;

// investment rate
jy = ss_p;
sy = ss_p;
j = ss_p * y_p;
jn = sn_p * yn_p;
jt = st_p * yt_p;

in = omegain_p *j;
ih = (1-omegain_p) * j;


// Labor
hw = hw_p;
hwt = hwt_p;
hwn = hwn_p;

// Output
gdp = y_p;
yn = yn_p;
yt = yt_p;

// Consumption
cc = gdp - j;
ccn = omegacn_p * cc;
cch = (1-omegacn_p) * cc;

// relative prices

pn = 1;
pc = 1;

pj = 1;


// Capital 
kpt = jt/delta_p;
kpn = jn/delta_p;

// Rental rate for labor
wn = (1-alphan_p)*yn_p/hwn_p;
wt = (1-alphat_p)*yt_p/hwt_p;

// Multipliers
lambda = 1/cc;

gamman = lambda;
gammat = lambda;

// Labor-augmenting technology
nn=1;
nt=1;

eta = lambda*wt;

end;

shocks;
var nnerr; stderr .01;
var nterr; stderr .01;
end;


stoch_simul(order=1,nocorr,nomoments,irf=0);

//stoch_simul(order=1,nocorr,nomoments,irf=100) yn yt hwn hwt ccn cch;
